@@ -298,7 +298,7 @@ void log_flush(int log_id)
 
 void log_reopen(int log_id)
 {
-    if (log_id < 0 || log_id >= LOG_MAXLOGS)
+    if (log_id < 0 && log_id >= LOG_MAXLOGS)
         return;
     if (loglist [log_id] . filename && loglist [log_id] . logfile)
     {
@@ -360,16 +360,17 @@ void log_shutdown(void)
 
 static int create_log_entry (int log_id, const char *pre, const char *line)
 {
+    int len;
     log_entry_t *entry;
 
     if (loglist[log_id].keep_entries == 0)
         return fprintf (loglist[log_id].logfile, "%s%s\n", pre, line); 
     
     entry = calloc (1, sizeof (log_entry_t));
-    entry->len = strlen (pre) + strlen (line) + 2;
-    entry->line = malloc (entry->len);
-    snprintf (entry->line, entry->len, "%s%s\n", pre, line);
-    loglist [log_id].total += entry->len;
+    entry->line = malloc (LOG_MAXLINELEN);
+    len = snprintf (entry->line, LOG_MAXLINELEN, "%s%s\n", pre, line);
+    entry->len = len;
+    loglist [log_id].total += len;
     fprintf (loglist[log_id].logfile, "%s", entry->line);
 
     *loglist [log_id].log_tail = entry;
@@ -385,7 +386,7 @@ static int create_log_entry (int log_id, const char *pre, const char *line)
     }
     else
         loglist [log_id].entries++;
-    return entry->len;
+    return len;
 }
 
 
@@ -419,138 +420,11 @@ void log_contents (int log_id, char **_contents, unsigned int *_len)
     _unlock_logger ();
 }
 
-static void __vsnprintf(char *str, size_t size, const char *format, va_list ap) {
-    int in_block = 0;
-    int block_size = 0;
-    int block_len;
-    int block_space = 0;
-    const char * arg;
-    char buf[80];
-
-    for (; *format && size; format++)
-    {
-        if ( !in_block )
-        {
-            if ( *format == '%' ) {
-                in_block = 1;
-                block_size = 0;
-                block_len  = 0;
-                block_space = 0;
-            }
-            else
-            {
-                *(str++) = *format;
-                size--;
-            }
-        }
-        else
-        {
-            // TODO: %l*[sdupi] as well as %.4080s and "%.*s
-            arg = NULL;
-            switch (*format)
-            {
-                case 'l':
-                    block_size++;
-                    break;
-                case '.':
-                    // just ignore '.'. If somebody cares: fix it.
-                    break;
-                case '*':
-                    block_len = va_arg(ap, int);
-                    break;
-                case ' ':
-                    block_space = 1;
-                    break;
-                case '1':
-                case '2':
-                case '3':
-                case '4':
-                case '5':
-                case '6':
-                case '7':
-                case '8':
-                case '9':
-                    block_len = atoi(format);
-                    for (; *format >= '0' && *format <= '9'; format++);
-                    format--;
-                    break;
-                case 'p':
-                    snprintf(buf, sizeof(buf), "%p", (void*)va_arg(ap, void *));
-                    arg = buf;
-                case 'd':
-                case 'i':
-                case 'u':
-                    if (!arg)
-                    {
-                        switch (block_size)
-                        {
-                            case 0:
-                                if (*format == 'u')
-                                    snprintf(buf, sizeof(buf), "%u", (unsigned int)va_arg(ap, unsigned int));
-                                else
-                                    snprintf(buf, sizeof(buf), "%i", (int)va_arg(ap, int));
-                                break;
-                            case 1:
-                                if (*format == 'u')
-                                    snprintf(buf, sizeof(buf), "%lu", (unsigned long int)va_arg(ap, unsigned long int));
-                                else
-                                    snprintf(buf, sizeof(buf), "%li", (long int)va_arg(ap, long int));
-                                break;
-                            case 2:
-                                if (*format == 'u')
-                                    snprintf(buf, sizeof(buf), "%llu", (unsigned long long int)va_arg(ap, unsigned long long int));
-                                else
-                                    snprintf(buf, sizeof(buf), "%lli", (long long int)va_arg(ap, long long int));
-                                break;
-                            default:
-                                snprintf(buf, sizeof(buf), "<<<invalid>>>");
-                                break;
-                        }
-                        arg = buf;
-                    }
-                case 's':
-                case 'H':
-                    // TODO.
-                    if (!arg)
-                        arg = va_arg(ap, const char *);
-                    if (!arg)
-                        arg = "(null)";
-                    if (!block_len)
-                        block_len = strlen(arg);
-
-                    // the if() is the outer structure so the inner for()
-                    // is branch optimized.
-                    if (*format == 'H' )
-                    {
-                        for (; *arg && block_len && size; arg++, size--, block_len--)
-                        {
-                            if ((*arg <= '"' || *arg == '`'  || *arg == '\\') && !(block_space && *arg == ' '))
-                                *(str++) = '.';
-                            else
-                                *(str++) = *arg;
-                        }
-                    }
-                    else
-                    {
-                        for (; *arg && block_len && size; arg++, size--, block_len--)
-                            *(str++) = *arg;
-                    }
-                    in_block = 0;
-                    break;
-            }
-        }
-    }
-
-    if ( !size )
-        str--;
-
-    *str = 0;
-}
 
 void log_write(int log_id, unsigned priority, const char *cat, const char *func, 
         const char *fmt, ...)
 {
-    static const char *prior[] = { "EROR", "WARN", "INFO", "DBUG" };
+    static char *prior[] = { "EROR", "WARN", "INFO", "DBUG" };
     int datelen;
     time_t now;
     char pre[256];
@@ -559,18 +433,18 @@ void log_write(int log_id, unsigned priority, const char *cat, const char *func,
 
     if (log_id < 0 || log_id >= LOG_MAXLOGS) return; /* Bad log number */
     if (loglist[log_id].level < priority) return;
-    if (!priority || priority > sizeof(prior)/sizeof(prior[0])) return; /* Bad priority */
-
+    if (priority > sizeof(prior)/sizeof(prior[0])) return; /* Bad priority */
 
     va_start(ap, fmt);
-    __vsnprintf(line, sizeof(line), fmt, ap);
-    va_end(ap);
+    vsnprintf(line, LOG_MAXLINELEN, fmt, ap);
 
     now = time(NULL);
-    datelen = strftime (pre, sizeof (pre), "[%Y-%m-%d  %H:%M:%S]", localtime(&now)); 
-    snprintf (pre+datelen, sizeof (pre)-datelen, " %s %s%s ", prior [priority-1], cat, func);
 
     _lock_logger();
+    datelen = strftime (pre, sizeof (pre), "[%Y-%m-%d  %H:%M:%S]", localtime(&now)); 
+
+    snprintf (pre+datelen, sizeof (pre)-datelen, " %s %s%s ", prior [priority-1], cat, func);
+
     if (_log_open (log_id))
     {
         int len = create_log_entry (log_id, pre, line);
@@ -578,13 +452,15 @@ void log_write(int log_id, unsigned priority, const char *cat, const char *func,
             loglist[log_id].size += len;
     }
     _unlock_logger();
+
+    va_end(ap);
 }
 
 void log_write_direct(int log_id, const char *fmt, ...)
 {
+    char line[LOG_MAXLINELEN];
     va_list ap;
     time_t now;
-    char line[LOG_MAXLINELEN];
 
     if (log_id < 0 || log_id >= LOG_MAXLOGS) return;
     
@@ -593,7 +469,7 @@ void log_write_direct(int log_id, const char *fmt, ...)
     now = time(NULL);
 
     _lock_logger();
-    __vsnprintf(line, LOG_MAXLINELEN, fmt, ap);
+    vsnprintf(line, LOG_MAXLINELEN, fmt, ap);
     if (_log_open (log_id))
     {
         int len = create_log_entry (log_id, "", line);
