@@ -131,7 +131,7 @@ int sock_error(void)
 #endif
 }
 
-void sock_set_error(int val)
+static void sock_set_error(int val)
 {
 #ifdef _WIN32
      WSASetLastError (val);
@@ -217,7 +217,7 @@ int sock_active (sock_t sock)
     l = recv (sock, &c, 1, MSG_PEEK);
     if (l == 0)
         return 0;
-    if (l == SOCK_ERROR && sock_recoverable (sock_error()))
+    if (l < 0 && sock_recoverable (sock_error()))
         return 1;
     return 0;
 }
@@ -242,18 +242,18 @@ int inet_aton(const char *s, struct in_addr *a)
 #endif /* _WIN32 */
 
 /* sock_set_blocking
- *
- * set the sock blocking or nonblocking
- * 1 for blocking
- * 0 for nonblocking
- */
-int sock_set_blocking(sock_t sock, int block)
+**
+** set the sock blocking or nonblocking
+** SOCK_BLOCK for blocking
+** SOCK_NONBLOCK for nonblocking
+*/
+int sock_set_blocking(sock_t sock, const int block)
 {
 #ifdef _WIN32
 #ifdef __MINGW32__
-    u_long varblock = 1;
+    u_long varblock = block;
 #else
-    int varblock = 1;
+    int varblock = block;
 #endif
 #endif
 
@@ -261,10 +261,9 @@ int sock_set_blocking(sock_t sock, int block)
         return SOCK_ERROR;
 
 #ifdef _WIN32
-    if (block) varblock = 0;
     return ioctlsocket(sock, FIONBIO, &varblock);
 #else
-    return fcntl(sock, F_SETFL, (block) ? 0 : O_NONBLOCK);
+    return fcntl(sock, F_SETFL, (block == SOCK_BLOCK) ? 0 : O_NONBLOCK);
 #endif
 }
 
@@ -516,27 +515,17 @@ int sock_read_line(sock_t sock, char *buff, const int len)
 int sock_connected (sock_t sock, int timeout)
 {
     struct pollfd check;
-    int val = SOCK_ERROR;
-    socklen_t size = sizeof val;
 
     check.fd = sock;
     check.events = POLLOUT;
     switch (poll (&check, 1, timeout*1000))
     {
         case 0: return SOCK_TIMEOUT;
-        default:
-            /* on windows getsockopt.val is defined as char* */
-            if (getsockopt(sock, SOL_SOCKET, SO_ERROR, (void*) &val, &size) == 0)
-            {
-                if (val == 0)
-                    return 1;
-                sock_set_error (val);
-            }
-            /* fall through */
         case -1:
             if (sock_recoverable (sock_error()))
                 return 0;
             return SOCK_ERROR;
+        default: return 1;
     }                                           
 }
 
@@ -581,11 +570,6 @@ int sock_connected (sock_t sock, int timeout)
 }
 #endif
 
-sock_t sock_connect_wto (const char *hostname, int port, int timeout)
-{
-    return sock_connect_wto_bind(hostname, port, NULL, timeout);
-}
-
 #ifdef HAVE_GETADDRINFO
 
 sock_t sock_connect_non_blocking (const char *hostname, unsigned port)
@@ -609,7 +593,7 @@ sock_t sock_connect_non_blocking (const char *hostname, unsigned port)
         if ((sock = socket (ai->ai_family, ai->ai_socktype, ai->ai_protocol)) 
                 > -1)
         {
-            sock_set_blocking (sock, 0);
+            sock_set_blocking (sock, SOCK_NONBLOCK);
             if (connect(sock, ai->ai_addr, ai->ai_addrlen) < 0 && 
                     !sock_connect_pending(sock_error()))
             {
@@ -630,10 +614,10 @@ sock_t sock_connect_non_blocking (const char *hostname, unsigned port)
  * timeout is 0 or less then we will wait until the OS gives up on the connect
  * The socket is returned
  */
-sock_t sock_connect_wto_bind (const char *hostname, int port, const char *bnd, int timeout)
+sock_t sock_connect_wto(const char *hostname, int port, int timeout)
 {
     sock_t sock = SOCK_ERROR;
-    struct addrinfo *ai, *head, *b_head=NULL, hints;
+    struct addrinfo *ai, *head, hints;
     char service[8];
 
     memset (&hints, 0, sizeof (hints));
@@ -650,23 +634,7 @@ sock_t sock_connect_wto_bind (const char *hostname, int port, const char *bnd, i
         if ((sock = socket (ai->ai_family, ai->ai_socktype, ai->ai_protocol)) >= 0)
         {
             if (timeout > 0)
-                sock_set_blocking (sock, 0);
-
-            if (bnd)
-            {
-                struct addrinfo b_hints;
-                memset (&b_hints, 0, sizeof(b_hints));
-                b_hints.ai_family = ai->ai_family;
-                b_hints.ai_socktype = ai->ai_socktype;
-                b_hints.ai_protocol = ai->ai_protocol;
-                if (getaddrinfo (bnd, NULL, &b_hints, &b_head) ||
-                        bind (sock, b_head->ai_addr, b_head->ai_addrlen) < 0)
-                {
-                    sock_close (sock);
-                    sock = SOCK_ERROR;
-                    break;
-                }
-            }
+                sock_set_blocking (sock, SOCK_NONBLOCK);
 
             if (connect (sock, ai->ai_addr, ai->ai_addrlen) == 0)
                 break;
@@ -682,7 +650,7 @@ sock_t sock_connect_wto_bind (const char *hostname, int port, const char *bnd, i
                     if (connected == 1) /* connected */
                     {
                         if (timeout >= 0)
-                            sock_set_blocking(sock, 1);
+                            sock_set_blocking(sock, SOCK_BLOCK);
                         break;
                     }
                 }
@@ -694,9 +662,8 @@ sock_t sock_connect_wto_bind (const char *hostname, int port, const char *bnd, i
         }
         ai = ai->ai_next;
     }
-    if (b_head)
-        freeaddrinfo (b_head);
-    freeaddrinfo (head);
+    if (head)
+        freeaddrinfo (head);
 
     return sock;
 }
@@ -780,7 +747,7 @@ int sock_try_connection (sock_t sock, const char *hostname, unsigned int port)
     memcpy(&server.sin_addr, &sin.sin_addr, sizeof(struct sockaddr_in));
 
     server.sin_family = AF_INET;
-    server.sin_port = htons((short)port);
+    server.sin_port = htons(port);
 
     return connect(sock, (struct sockaddr *)&server, sizeof(server));
 }
@@ -793,13 +760,13 @@ sock_t sock_connect_non_blocking (const char *hostname, unsigned port)
     if (sock == SOCK_ERROR)
         return SOCK_ERROR;
 
-    sock_set_blocking (sock, 0);
+    sock_set_blocking (sock, SOCK_NONBLOCK);
     sock_try_connection (sock, hostname, port);
     
     return sock;
 }
 
-sock_t sock_connect_wto_bind (const char *hostname, int port, const char *bnd, int timeout)
+sock_t sock_connect_wto(const char *hostname, int port, int timeout)
 {
     sock_t sock;
 
@@ -807,24 +774,9 @@ sock_t sock_connect_wto_bind (const char *hostname, int port, const char *bnd, i
     if (sock == SOCK_ERROR)
         return SOCK_ERROR;
 
-    if (bnd)
-    {
-        struct sockaddr_in sa;
-
-        memset(&sa, 0, sizeof(sa));
-        sa.sin_family = AF_INET;
-
-        if (inet_aton (bnd, &sa.sin_addr) == 0 ||
-            bind (sock, (struct sockaddr *)&sa, sizeof(sa)) < 0)
-        {
-            sock_close (sock);
-            return SOCK_ERROR;
-        }
-    }
-
     if (timeout)
     {
-        sock_set_blocking (sock, 0);
+        sock_set_blocking (sock, SOCK_NONBLOCK);
         if (sock_try_connection (sock, hostname, port) < 0)
         {
             int ret = sock_connected (sock, timeout);
@@ -834,7 +786,7 @@ sock_t sock_connect_wto_bind (const char *hostname, int port, const char *bnd, i
                 return SOCK_ERROR;
             }
         }
-        sock_set_blocking(sock, 1);
+        sock_set_blocking(sock, SOCK_BLOCK);
     }
     else
     {
@@ -876,12 +828,12 @@ sock_t sock_get_server_socket(int port, const char *sinterface)
             return SOCK_ERROR;
         } else {
             sa.sin_family = AF_INET;
-            sa.sin_port = htons((short)port);
+            sa.sin_port = htons(port);
         }
     } else {
         sa.sin_addr.s_addr = INADDR_ANY;
         sa.sin_family = AF_INET;
-        sa.sin_port = htons((short)port);
+        sa.sin_port = htons(port);
     }
 
     /* get a socket */
@@ -903,11 +855,6 @@ sock_t sock_get_server_socket(int port, const char *sinterface)
 
 #endif
 
-void sock_set_send_buffer (sock_t sock, int win_size)
-{
-    setsockopt (sock, SOL_SOCKET, SO_SNDBUF, (char *) &win_size, sizeof(win_size));
-}
-
 int sock_listen(sock_t serversock, int backlog)
 {
     if (!sock_valid_socket(serversock))
