@@ -3,12 +3,11 @@
  * This program is distributed under the GNU General Public License, version 2.
  * A copy of this license is included with this source.
  *
- * Copyright 2000-2004, Jack Moffitt <jack@xiph.org,
+ * Copyright 2000-2004, Jack Moffitt <jack@xiph.org, 
  *                      Michael Smith <msmith@xiph.org>,
  *                      oddsock <oddsock@xiph.org>,
  *                      Karl Heyes <karl@xiph.org>
  *                      and others (see AUTHORS for details).
- * Copyright 2012-2022, Philipp "ph3-der-loewe" Schafft <lion@lion.leolix.org>,
  */
 
 #ifdef HAVE_CONFIG_H
@@ -19,32 +18,29 @@
 #include <string.h>
 #include <stdlib.h>
 #include <stdarg.h>
-#include <ctype.h>
 
 #include <libxml/xmlmemory.h>
 #include <libxml/parser.h>
 #include <libxml/tree.h>
 
-#include "common/thread/thread.h"
-#include "common/avl/avl.h"
-#include "common/httpp/httpp.h"
-#include "common/net/sock.h"
+#include "thread/thread.h"
+#include "avl/avl.h"
+#include "httpp/httpp.h"
+#include "net/sock.h"
 
-#include "stats.h"
 #include "connection.h"
+
 #include "source.h"
 #include "global.h"
 #include "refbuf.h"
 #include "client.h"
-#include "admin.h"
+#include "stats.h"
 #include "xslt.h"
 #include "util.h"
-#include "auth.h"
 #define CATMODULE "stats"
 #include "logging.h"
 
 #ifdef _WIN32
-#define atoll _atoi64
 #define vsnprintf _vsnprintf
 #define snprintf _snprintf
 #endif
@@ -87,16 +83,15 @@ static volatile event_listener_t *_event_listeners;
 
 
 static void *_stats_thread(void *arg);
-static int _compare_stats(void *arg, void *a, void *b);
-static int _compare_source_stats(void *arg, void *a, void *b);
+static int _compare_stats(void *a, void *b, void *arg);
+static int _compare_source_stats(void *a, void *b, void *arg);
 static int _free_stats(void *key);
 static int _free_source_stats(void *key);
 static void _add_event_to_queue(stats_event_t *event, event_queue_t *queue);
 static stats_node_t *_find_node(avl_tree *tree, const char *name);
 static stats_source_t *_find_source(avl_tree *tree, const char *source);
 static void _free_event(stats_event_t *event);
-static stats_event_t *_get_event_from_queue(event_queue_t *queue);
-static void __add_metadata(xmlNodePtr node, const char *tag);
+static stats_event_t *_get_event_from_queue (event_queue_t *queue);
 
 
 /* simple helper function for creating an event */
@@ -138,7 +133,7 @@ void stats_initialize(void)
     thread_mutex_create(&_stats_mutex);
 
     /* set up stats queues */
-    event_queue_init(&_global_event_queue);
+    event_queue_init (&_global_event_queue);
     thread_mutex_create(&_global_event_mutex);
 
     /* fire off the stats thread */
@@ -150,13 +145,11 @@ void stats_shutdown(void)
 {
     int n;
 
-    if (!_stats_running) /* We can't shutdown if we're not running. */
+    if(!_stats_running) /* We can't shutdown if we're not running. */
         return;
 
     /* wait for thread to exit */
-    thread_mutex_lock(&_stats_mutex);
     _stats_running = 0;
-    thread_mutex_unlock(&_stats_mutex);
     thread_join(_stats_thread_id);
 
     /* wait for other threads to shut down */
@@ -166,7 +159,7 @@ void stats_shutdown(void)
         n = _stats_threads;
         thread_mutex_unlock(&_stats_mutex);
     } while (n > 0);
-    ICECAST_LOG_INFO("stats thread finished");
+    INFO0("stats thread finished");
 
     /* free the queues */
 
@@ -194,7 +187,7 @@ void stats_shutdown(void)
 stats_t *stats_get_stats(void)
 {
     /* lock global stats
-
+    
      copy stats
 
      unlock global stats
@@ -211,12 +204,12 @@ void stats_event(const char *source, const char *name, const char *value)
 
     if (value && xmlCheckUTF8 ((unsigned char *)value) == 0)
     {
-        ICECAST_LOG_WARN("seen non-UTF8 data, probably incorrect metadata (%s, %s)", name, value);
+        WARN2 ("seen non-UTF8 data, probably incorrect metadata (%s, %s)", name, value);
         return;
     }
-    event = build_event(source, name, value);
+    event = build_event (source, name, value);
     if (event)
-        queue_global_event(event);
+        queue_global_event (event);
 }
 
 
@@ -240,7 +233,7 @@ void stats_event_conv(const char *mount, const char *name, const char *value, co
             xmlCharEncCloseFunc (handle);
         }
         else
-            ICECAST_LOG_WARN("No charset found for \"%s\"", charset);
+            WARN1 ("No charset found for \"%s\"", charset);
     }
 
     stats_event (mount, name, metadata);
@@ -274,12 +267,12 @@ void stats_event_args(const char *source, char *name, char *format, ...)
     if (name == NULL)
         return;
     va_start(val, format);
-    ret = vsnprintf(buf, sizeof(buf), format, val);
+    ret = vsnprintf(buf, 1024, format, val);
     va_end(val);
 
     if (ret < 0 || (unsigned int)ret >= sizeof (buf))
     {
-        ICECAST_LOG_WARN("problem with formatting %s stat %s",
+        WARN2 ("problem with formatting %s stat %s",
                 source==NULL ? "global" : source, name);
         return;
     }
@@ -319,7 +312,7 @@ char *stats_get_value(const char *source, const char *name)
 void stats_event_inc(const char *source, const char *name)
 {
     stats_event_t *event = build_event (source, name, NULL);
-    /* ICECAST_LOG_DEBUG("%s on %s", name, source==NULL?"global":source); */
+    /* DEBUG2("%s on %s", name, source==NULL?"global":source); */
     if (event)
     {
         event->action = STATS_EVENT_INC;
@@ -330,11 +323,11 @@ void stats_event_inc(const char *source, const char *name)
 void stats_event_add(const char *source, const char *name, unsigned long value)
 {
     stats_event_t *event = build_event (source, name, NULL);
-    /* ICECAST_LOG_DEBUG("%s on %s", name, source==NULL?"global":source); */
+    /* DEBUG2("%s on %s", name, source==NULL?"global":source); */
     if (event)
     {
         event->value = malloc (16);
-        snprintf(event->value, 16, "%lu", value);
+        snprintf (event->value, 16, "%ld", value);
         event->action = STATS_EVENT_ADD;
         queue_global_event (event);
     }
@@ -346,7 +339,7 @@ void stats_event_sub(const char *source, const char *name, unsigned long value)
     if (event)
     {
         event->value = malloc (16);
-        snprintf(event->value, 16, "%lu", value);
+        snprintf (event->value, 16, "%ld", value);
         event->action = STATS_EVENT_SUB;
         queue_global_event (event);
     }
@@ -355,7 +348,7 @@ void stats_event_sub(const char *source, const char *name, unsigned long value)
 /* decrease the value in the provided stat by 1 */
 void stats_event_dec(const char *source, const char *name)
 {
-    /* ICECAST_LOG_DEBUG("%s on %s", name, source==NULL?"global":source); */
+    /* DEBUG2("%s on %s", name, source==NULL?"global":source); */
     stats_event_t *event = build_event (source, name, NULL);
     if (event)
     {
@@ -375,18 +368,18 @@ static stats_node_t *_find_node(avl_tree *stats_tree, const char *name)
 
     /* get the root node */
     node = stats_tree->root->right;
-
+    
     while (node) {
-        stats = (stats_node_t *) node->key;
+        stats = (stats_node_t *)node->key;
         cmp = strcmp(name, stats->name);
-        if (cmp < 0)
+        if (cmp < 0) 
             node = node->left;
         else if (cmp > 0)
             node = node->right;
         else
             return stats;
     }
-
+    
     /* didn't find it */
     return NULL;
 }
@@ -403,7 +396,7 @@ static stats_source_t *_find_source(avl_tree *source_tree, const char *source)
     /* get the root node */
     node = source_tree->root->right;
     while (node) {
-        stats = (stats_source_t *) node->key;
+        stats = (stats_source_t *)node->key;
         cmp = strcmp(source, stats->source);
         if (cmp < 0)
             node = node->left;
@@ -420,7 +413,7 @@ static stats_source_t *_find_source(avl_tree *source_tree, const char *source)
 static stats_event_t *_copy_event(stats_event_t *event)
 {
     stats_event_t *copy = (stats_event_t *)calloc(1, sizeof(stats_event_t));
-    if (event->source)
+    if (event->source) 
         copy->source = (char *)strdup(event->source);
     else
         copy->source = NULL;
@@ -438,20 +431,21 @@ static stats_event_t *_copy_event(stats_event_t *event)
 
 
 /* helper to apply specialised changes to a stats node */
-static void modify_node_event(stats_node_t *node, stats_event_t *event)
+static void modify_node_event (stats_node_t *node, stats_event_t *event)
 {
     char *str;
 
-    if (event->action == STATS_EVENT_HIDDEN) {
+    if (event->action == STATS_EVENT_HIDDEN)
+    {
         if (event->value)
             node->hidden = 1;
         else
             node->hidden = 0;
         return;
     }
-
-    if (event->action != STATS_EVENT_SET) {
-        int64_t value = 0;
+    if (event->action != STATS_EVENT_SET)
+    {
+        int value = 0;
 
         switch (event->action)
         {
@@ -464,27 +458,19 @@ static void modify_node_event(stats_node_t *node, stats_event_t *event)
             case STATS_EVENT_ADD:
                 value = atoi (node->value)+atoi (event->value);
                 break;
-            case STATS_EVENT_SUB:
-                value = atoll (node->value) - atoll (event->value);
-                break;
             default:
-                ICECAST_LOG_WARN("unhandled event (%d) for %s", event->action, event->source);
                 break;
         }
-        str = malloc(16);
-        snprintf(str, 16, "%" PRId64, value);
+        str = malloc (16);
+        snprintf (str, 16, "%d", value);
         if (event->value == NULL)
-            event->value = strdup(str);
-    } else {
-        str = (char *)strdup(event->value);
+            event->value = strdup (str);
     }
-    free(node->value);
+    else
+        str = (char *)strdup (event->value);
+    free (node->value);
     node->value = str;
-    if (event->source) {
-        ICECAST_LOG_DEBUG("update \"%s\" %s (%s)", event->source, node->name, node->value);
-    } else {
-        ICECAST_LOG_DEBUG("update global %s (%s)", node->name, node->value);
-    }
+    DEBUG2 ("update node %s (%s)", node->name, node->value);
 }
 
 
@@ -492,7 +478,7 @@ static void process_global_event (stats_event_t *event)
 {
     stats_node_t *node;
 
-    /* ICECAST_LOG_DEBUG("global event %s %s %d", event->name, event->value, event->action); */
+    /* DEBUG3("global event %s %s %d", event->name, event->value, event->action); */
     if (event->action == STATS_EVENT_REMOVE)
     {
         /* we're deleting */
@@ -528,7 +514,7 @@ static void process_source_event (stats_event_t *event)
         snode = (stats_source_t *)calloc(1,sizeof(stats_source_t));
         if (snode == NULL)
             return;
-        ICECAST_LOG_DEBUG("new source stat %#H", event->source);
+        DEBUG1 ("new source stat %s", event->source);
         snode->source = (char *)strdup(event->source);
         snode->stats_tree = avl_tree_new(_compare_stats, NULL);
         if (event->action == STATS_EVENT_HIDDEN)
@@ -536,17 +522,19 @@ static void process_source_event (stats_event_t *event)
         else
             snode->hidden = 0;
 
-        avl_insert(_stats.source_tree, (void *) snode);
+        avl_insert(_stats.source_tree, (void *)snode);
     }
-
-    if (event->name) {
+    if (event->name)
+    {
         stats_node_t *node = _find_node(snode->stats_tree, event->name);
-        if (node == NULL) {
+        if (node == NULL)
+        {
             if (event->action == STATS_EVENT_REMOVE)
                 return;
             /* adding node */
-            if (event->value) {
-                ICECAST_LOG_DEBUG("new node %H on %#H (% H)", event->name, event->source, event->value);
+            if (event->value)
+            {
+                DEBUG2 ("new node %s (%s)", event->name, event->value);
                 node = (stats_node_t *)calloc(1,sizeof(stats_node_t));
                 node->name = (char *)strdup(event->name);
                 node->value = (char *)strdup(event->value);
@@ -556,114 +544,57 @@ static void process_source_event (stats_event_t *event)
             }
             return;
         }
-
-        if (event->action == STATS_EVENT_REMOVE) {
-            ICECAST_LOG_DEBUG("delete node %s", event->name);
+        if (event->action == STATS_EVENT_REMOVE)
+        {
+            DEBUG1 ("delete node %s", event->name);
             avl_delete(snode->stats_tree, (void *)node, _free_stats);
             return;
         }
-        modify_node_event(node, event);
+        modify_node_event (node, event);
         return;
     }
-
-    if (event->action == STATS_EVENT_HIDDEN) {
+    if (event->action == STATS_EVENT_HIDDEN)
+    {
         avl_node *node = avl_get_first (snode->stats_tree);
 
-        if (event->value) {
+        if (event->value)
             snode->hidden = 1;
-        } else {
+        else
             snode->hidden = 0;
-        }
-
-        while (node) {
+        while (node)
+        {
             stats_node_t *stats = (stats_node_t*)node->key;
             stats->hidden = snode->hidden;
-            node = avl_get_next(node);
+            node = avl_get_next (node);
         }
-
         return;
     }
-
-    if (event->action == STATS_EVENT_REMOVE) {
-        ICECAST_LOG_DEBUG("delete source node %s", event->source);
+    if (event->action == STATS_EVENT_REMOVE)
+    {
+        DEBUG1 ("delete source node %s", event->source);
         avl_delete(_stats.source_tree, (void *)snode, _free_source_stats);
     }
 }
 
-/* NOTE: implicit %z is added to format string. */
-static inline void __format_time(char * buffer, size_t len, const char * format) {
-    time_t now = time(NULL);
-    struct tm local;
-    char tzbuffer[32];
-    char timebuffer[128];
-#ifdef _WIN32
-    struct tm *thetime;
-    int time_days, time_hours, time_tz;
-    int tempnum1, tempnum2;
-    char sign;
-#endif
-
-    localtime_r (&now, &local);
-#ifndef _WIN32
-    strftime (tzbuffer, sizeof(tzbuffer), "%z", &local);
-#else
-    thetime = gmtime (&now);
-    time_days = local.tm_yday - thetime->tm_yday;
-
-    if (time_days < -1) {
-        tempnum1 = 24;
-    } else {
-        tempnum1 = 1;
-    }
-
-    if (tempnum1 < time_days) {
-        tempnum2 = -24;
-    } else {
-        tempnum2 = time_days*24;
-    }
-
-    time_hours = (tempnum2 + local.tm_hour - thetime->tm_hour);
-    time_tz = time_hours * 60 + local.tm_min - thetime->tm_min;
-
-    if (time_tz < 0) {
-        sign = '-';
-        time_tz = -time_tz;
-    } else {
-        sign = '+';
-    }
-
-    snprintf(tzbuffer, sizeof(tzbuffer), "%c%.2d%.2d", sign, time_tz / 60, time_tz % 60);
-#endif
-    strftime(timebuffer, sizeof(timebuffer), format, &local);
-
-    snprintf(buffer, len, "%s%s", timebuffer, tzbuffer);
-}
 
 void stats_event_time (const char *mount, const char *name)
 {
-    char buffer[256];
+    time_t now = time(NULL);
+    struct tm local;
+    char buffer[100];
 
-    __format_time(buffer, sizeof(buffer), "%a, %d %b %Y %H:%M:%S ");
-    stats_event(mount, name, buffer);
-}
-
-
-void stats_event_time_iso8601 (const char *mount, const char *name)
-{
-    char buffer[256];
-
-    __format_time(buffer, sizeof(buffer), "%Y-%m-%dT%H:%M:%S");
-    stats_event(mount, name, buffer);
+    localtime_r (&now, &local);
+    strftime (buffer, sizeof (buffer), "%a, %d %b %Y %H:%M:%S %z", &local);
+    stats_event (mount, name, buffer);
 }
 
 
 void stats_global (ice_config_t *config)
 {
-    stats_event(NULL, "server_id", config->server_id);
-    stats_event(NULL, "host", config->hostname);
-    stats_event(NULL, "location", config->location);
-    stats_event(NULL, "admin", config->admin);
-    stats_event(NULL, "instance_uuid", global_instance_uuid());
+    stats_event (NULL, "server_id", config->server_id);
+    stats_event (NULL, "host", config->hostname);
+    stats_event (NULL, "location", config->location);
+    stats_event (NULL, "admin", config->admin);
 }
 
 
@@ -673,38 +604,28 @@ static void *_stats_thread(void *arg)
     stats_event_t *copy;
     event_listener_t *listener;
 
-    (void)arg;
-
-    stats_event_time(NULL, "server_start");
-    stats_event_time_iso8601(NULL, "server_start_iso8601");
+    stats_event_time (NULL, "server_start");
 
     /* global currently active stats */
-    stats_event(NULL, "clients", "0");
-    stats_event(NULL, "connections", "0");
-    stats_event(NULL, "sources", "0");
-    stats_event(NULL, "stats", "0");
-    stats_event(NULL, "listeners", "0");
+    stats_event (NULL, "clients", "0");
+    stats_event (NULL, "connections", "0");
+    stats_event (NULL, "sources", "0");
+    stats_event (NULL, "stats", "0");
+    stats_event (NULL, "listeners", "0");
 
     /* global accumulating stats */
-    stats_event(NULL, "client_connections", "0");
-    stats_event(NULL, "source_client_connections", "0");
-    stats_event(NULL, "source_relay_connections", "0");
-    stats_event(NULL, "source_total_connections", "0");
-    stats_event(NULL, "stats_connections", "0");
-    stats_event(NULL, "listener_connections", "0");
+    stats_event (NULL, "client_connections", "0");
+    stats_event (NULL, "source_client_connections", "0");
+    stats_event (NULL, "source_relay_connections", "0");
+    stats_event (NULL, "source_total_connections", "0");
+    stats_event (NULL, "stats_connections", "0");
+    stats_event (NULL, "listener_connections", "0");
 
-    ICECAST_LOG_INFO("stats thread started");
-    while (1) {
-        thread_mutex_lock(&_stats_mutex);
-        if (!_stats_running) {
-            thread_mutex_unlock(&_stats_mutex);
-            break;
-        }
-        thread_mutex_unlock(&_stats_mutex);
-
-        thread_mutex_lock(&_global_event_mutex);
+    INFO0 ("stats thread started");
+    while (_stats_running) {
         if (_global_event_queue.head != NULL) {
             /* grab the next event from the queue */
+            thread_mutex_lock(&_global_event_mutex);
             event = _get_event_from_queue (&_global_event_queue);
             thread_mutex_unlock(&_global_event_mutex);
 
@@ -716,10 +637,10 @@ static void *_stats_thread(void *arg)
 
             /* check if we are dealing with a global or source event */
             if (event->source == NULL)
-                process_global_event(event);
+                process_global_event (event);
             else
-                process_source_event(event);
-
+                process_source_event (event);
+            
             /* now we have an event that's been processed into the running stats */
             /* this event should get copied to event listeners' queues */
             listener = (event_listener_t *)_event_listeners;
@@ -738,10 +659,6 @@ static void *_stats_thread(void *arg)
             thread_mutex_unlock(&_stats_mutex);
             continue;
         }
-        else
-        {
-            thread_mutex_unlock(&_global_event_mutex);
-        }
 
         thread_sleep(300000);
     }
@@ -770,7 +687,7 @@ static void _unregister_listener(event_listener_t *listener)
 static stats_event_t *_make_event_from_node(stats_node_t *node, char *source)
 {
     stats_event_t *event = (stats_event_t *)malloc(sizeof(stats_event_t));
-
+    
     if (source != NULL)
         event->source = (char *)strdup(source);
     else
@@ -826,138 +743,43 @@ static int _send_event_to_client(stats_event_t *event, client_t *client)
     return 0;
 }
 
-void stats_add_authstack(auth_stack_t *stack, xmlNodePtr parent)
+void _dump_stats_to_queue (event_queue_t *queue)
 {
-    xmlNodePtr authentication;
-    authentication = xmlNewTextChild(parent, NULL, XMLSTR("authentication"), NULL);
-
-    auth_stack_addref(stack);
-
-    while (stack) {
-        auth_t *auth = auth_stack_get(stack);
-        admin_add_role_to_authentication(auth, authentication);
-        auth_release(auth);
-        auth_stack_next(&stack);
-   }
-}
-
-static inline int __is_in_list(const char *key, const char *list[])
-{
-    size_t i;
-    for (i = 0; list[i]; i++)
-        if (strcmp(key, list[i]) == 0)
-            return 1;
-    return 0;
-}
-
-static inline int __include_node(unsigned int flags, const char *key, const char *list[])
-{
-    return !(flags & STATS_XML_FLAG_PUBLIC_VIEW) || __is_in_list(key, list);
-}
-
-static xmlNodePtr _dump_stats_to_doc (xmlNodePtr root, unsigned int flags, const char *show_mount, client_t *client) {
-    static const char *public_keys_global[] = {"admin", "location", "host", "server_id", "server_start_iso8601", NULL};
-    static const char *public_keys_source[] = {"listeners", "server_name", "server_description", "stream_start_iso8601", "subtype", "content-type", "listenurl", "genre", "display-title", NULL};
-    int hidden = flags & STATS_XML_FLAG_SHOW_HIDDEN ? 1 : 0;
-    avl_node *avlnode;
-    xmlNodePtr ret = NULL;
-    ice_config_t *config;
-
-    if (flags & STATS_XML_FLAG_PUBLIC_VIEW) {
-        /* Ensure those flags are clear when rendering a public view */
-        flags &= ~(STATS_XML_FLAG_SHOW_LISTENERS|STATS_XML_FLAG_SHOW_HIDDEN);
-    } else {
-        config = config_get_config();
-        stats_add_authstack(config->authstack, root);
-        config_release_config();
-    }
+    avl_node *node;
+    avl_node *node2;
+    stats_event_t *event;
+    stats_source_t *source;
 
     thread_mutex_lock(&_stats_mutex);
-    /* general stats first */
-    avlnode = avl_get_first(_stats.global_tree);
+    /* first we fill our queue with the current stats */
+    /* start with the global stats */
+    node = avl_get_first(_stats.global_tree);
+    while (node) {
+        event = _make_event_from_node((stats_node_t *)node->key, NULL);
+        _add_event_to_queue(event, queue);
 
-    while (avlnode) {
-        stats_node_t *stat = avlnode->key;
-        if (stat->hidden <=  hidden && __include_node(flags, stat->name, public_keys_global))
-            xmlNewTextChild (root, NULL, XMLSTR(stat->name), XMLSTR(stat->value));
-        avlnode = avl_get_next (avlnode);
+        node = avl_get_next(node);
     }
-    /* now per mount stats */
-    avlnode = avl_get_first(_stats.source_tree);
 
-    while (avlnode) {
-        stats_source_t *source = (stats_source_t *)avlnode->key;
+    /* now the stats for each source */
+    node = avl_get_first(_stats.source_tree);
+    while (node) {
+        source = (stats_source_t *)node->key;
+        node2 = avl_get_first(source->stats_tree);
+        while (node2) {
+            event = _make_event_from_node((stats_node_t *)node2->key, source->source);
+            _add_event_to_queue(event, queue);
 
-        if (source->hidden <= hidden &&
-                (show_mount == NULL || strcmp (show_mount, source->source) == 0))
-        {
-            xmlNodePtr metadata, history;
-            source_t *source_real;
-            mount_proxy *mountproxy;
-            int i;
-
-            avl_node *avlnode2 = avl_get_first (source->stats_tree);
-            xmlNodePtr xmlnode = xmlNewTextChild (root, NULL, XMLSTR("source"), NULL);
-
-            xmlSetProp (xmlnode, XMLSTR("mount"), XMLSTR(source->source));
-            if (ret == NULL)
-                ret = xmlnode;
-            while (avlnode2)
-            {
-                stats_node_t *stat = avlnode2->key;
-                if (__include_node(flags, stat->name, public_keys_source)) {
-                    if (client && strcmp(stat->name, "listenurl") == 0) {
-                        char buf[512];
-                        client_get_baseurl(client, NULL, buf, sizeof(buf), NULL, NULL, NULL, source->source, NULL);
-                        xmlNewTextChild (xmlnode, NULL, XMLSTR(stat->name), XMLSTR(buf));
-                    } else {
-                        xmlNewTextChild (xmlnode, NULL, XMLSTR(stat->name), XMLSTR(stat->value));
-                    }
-                }
-                avlnode2 = avl_get_next (avlnode2);
-            }
-
-
-            avl_tree_rlock(global.source_tree);
-            source_real = source_find_mount_raw(source->source);
-            if (source_real) {
-                history = playlist_render_xspf(source_real->history);
-                if (history)
-                    xmlAddChild(xmlnode, history);
-
-                metadata = xmlNewTextChild(xmlnode, NULL, XMLSTR("metadata"), NULL);
-                if (source_real->format) {
-                    for (i = 0; i < source_real->format->vc.comments; i++)
-                        __add_metadata(metadata, source_real->format->vc.user_comments[i]);
-                }
-
-                if (source_real->running)
-                    xmlNewTextChild(xmlnode, NULL, XMLSTR("content-type"), XMLSTR(source_real->format->contenttype));
-
-                if (flags & STATS_XML_FLAG_SHOW_LISTENERS) {
-                    admin_add_listeners_to_mount(source_real, xmlnode, client->mode);
-                    admin_add_geoip_to_mount(source_real, xmlnode, client->mode);
-                }
-            }
-            avl_tree_unlock(global.source_tree);
-
-            if (!(flags & STATS_XML_FLAG_PUBLIC_VIEW)) {
-                config = config_get_config();
-                mountproxy = config_find_mount(config, source->source, MOUNT_TYPE_NORMAL);
-                if (mountproxy)
-                    stats_add_authstack(mountproxy->authstack, xmlnode);
-                config_release_config();
-            }
+            node2 = avl_get_next(node2);
         }
-        avlnode = avl_get_next (avlnode);
+        
+        node = avl_get_next(node);
     }
     thread_mutex_unlock(&_stats_mutex);
-    return ret;
 }
 
-
 /* factoring out code for stats loops
-** this function copies all stats to queue, and registers
+** this function copies all stats to queue, and registers 
 ** the queue for all new events atomically.
 ** note: mutex must already be created!
 */
@@ -971,12 +793,12 @@ static void _register_listener (event_listener_t *listener)
     thread_mutex_lock(&_stats_mutex);
 
     /* first we fill our queue with the current stats */
-
+    
     /* start with the global stats */
     node = avl_get_first(_stats.global_tree);
     while (node) {
-        event = _make_event_from_node((stats_node_t *) node->key, NULL);
-        _add_event_to_queue(event, &listener->queue);
+        event = _make_event_from_node((stats_node_t *)node->key, NULL);
+        _add_event_to_queue (event, &listener->queue);
 
         node = avl_get_next(node);
     }
@@ -992,7 +814,7 @@ static void _register_listener (event_listener_t *listener)
 
             node2 = avl_get_next(node2);
         }
-
+        
         node = avl_get_next(node);
     }
 
@@ -1009,7 +831,7 @@ void *stats_connection(void *arg)
     stats_event_t *event;
     event_listener_t listener;
 
-    ICECAST_LOG_INFO("stats client starting");
+    INFO0 ("stats client starting");
 
     event_queue_init (&listener.queue);
     /* increment the thread count */
@@ -1022,14 +844,7 @@ void *stats_connection(void *arg)
 
     _register_listener (&listener);
 
-    while (1) {
-        thread_mutex_lock(&_stats_mutex);
-        if (!_stats_running) {
-            thread_mutex_unlock(&_stats_mutex);
-            break;
-        }
-        thread_mutex_unlock(&_stats_mutex);
-
+    while (_stats_running) {
         thread_mutex_lock (&listener.mutex);
         event = _get_event_from_queue (&listener.queue);
         thread_mutex_unlock (&listener.mutex);
@@ -1052,7 +867,7 @@ void *stats_connection(void *arg)
 
     thread_mutex_destroy (&listener.mutex);
     client_destroy (client);
-    ICECAST_LOG_INFO("stats client finished");
+    INFO0 ("stats client finished");
 
     return NULL;
 }
@@ -1060,8 +875,6 @@ void *stats_connection(void *arg)
 
 void stats_callback (client_t *client, void *notused)
 {
-    (void)notused;
-
     if (client->con->error)
     {
         client_destroy (client);
@@ -1079,70 +892,170 @@ typedef struct _source_xml_tag {
     struct _source_xml_tag *next;
 } source_xml_t;
 
+static xmlNodePtr _find_xml_node(char *mount, source_xml_t **list, xmlNodePtr root)
+{
+    source_xml_t *node, *node2;
+    int found = 0;
 
-void stats_transform_xslt(client_t *client)
+    /* search for existing node */
+    node = *list;
+    while (node) {
+        if (strcmp(node->mount, mount) == 0) {
+            found = 1;
+            break;
+        }
+        node = node->next;
+    }
+
+    if (found) return node->node;
+
+    /* if we didn't find it, we must build it and add it to the list */
+
+    /* build node */
+    node = (source_xml_t *)malloc(sizeof(source_xml_t));
+    node->mount = strdup(mount);
+    node->node = xmlNewChild (root, NULL, XMLSTR("source"), NULL);
+    xmlSetProp (node->node, XMLSTR("mount"), XMLSTR(mount));
+    node->next = NULL;
+
+    /* add node */
+    if (*list == NULL) {
+        *list = node;
+    } else {
+        node2 = *list;
+        while (node2->next) node2 = node2->next;
+        node2->next = node;
+    }
+
+    return node->node;
+}
+
+void stats_transform_xslt(client_t *client, const char *uri)
 {
     xmlDocPtr doc;
-    char *xslpath = util_get_path_from_normalised_uri(client->uri);
-    const char *mount = httpp_get_param(client->parser, "mount");
+    char *xslpath = util_get_path_from_normalised_uri (uri);
+    const char *mount = httpp_get_query_param (client->parser, "mount");
 
-    doc = stats_get_xml(STATS_XML_FLAG_NONE, mount, client);
+    stats_get_xml(&doc, 0, mount);
 
-    xslt_transform(doc, xslpath, client, 200, NULL, NULL);
+    xslt_transform(doc, xslpath, client);
 
     xmlFreeDoc(doc);
-    free(xslpath);
+    free (xslpath);
 }
 
-static void __add_metadata(xmlNodePtr node, const char *tag) {
-    const char *value = strstr(tag, "=");
-    char *name = NULL;
-    size_t namelen = value - tag + 1;
-    size_t i;
-
-    if (!value)
-        return;
-
-    name = malloc(namelen);
-    if (!name)
-        return;
-
-    for (i = 0; i < (namelen - 1); i++)
-        name[i] = tolower(tag[i]);
-
-    name[namelen-1] = 0;
-
-    xmlNewTextChild(node, NULL, XMLSTR(name), XMLSTR(value+1));
-
-    free(name);
-}
-
-xmlDocPtr stats_get_xml(unsigned int flags, const char *show_mount, client_t *client)
+void stats_get_xml(xmlDocPtr *doc, int show_hidden, const char *show_mount)
 {
+    stats_event_t *event;
+    event_queue_t queue;
+    xmlNodePtr node, srcnode;
+    source_xml_t *src_nodes = NULL;
+    source_xml_t *next;
+
+    event_queue_init (&queue);
+    _dump_stats_to_queue (&queue);
+
+    *doc = xmlNewDoc (XMLSTR("1.0"));
+    node = xmlNewDocNode(*doc, NULL, XMLSTR("icestats"), NULL);
+    xmlDocSetRootElement(*doc, node);
+
+    event = _get_event_from_queue(&queue);
+    while (event)
+    {
+        if (event->hidden <= show_hidden)
+        {
+            do
+            {
+                xmlChar *name, *value;
+                name = xmlEncodeEntitiesReentrant (*doc, XMLSTR(event->name));
+                value = xmlEncodeEntitiesReentrant (*doc, XMLSTR(event->value));
+                srcnode = node;
+                if (event->source)
+                {
+                    if (show_mount && strcmp (event->source, show_mount) != 0)
+                        break;
+                    srcnode = _find_xml_node(event->source, &src_nodes, node);
+                }
+                else
+                    srcnode = node;
+                xmlNewChild(srcnode, NULL, XMLSTR(name), XMLSTR(value));
+                xmlFree (value);
+                xmlFree (name);
+            } while (0);
+        }
+
+        _free_event(event);
+        event = _get_event_from_queue(&queue);
+    }
+
+    while (src_nodes) {
+        next = src_nodes->next;
+        free(src_nodes->mount);
+        free(src_nodes);
+        src_nodes = next;
+    }
+}
+void stats_sendxml(client_t *client)
+{
+    int bytes;
+    stats_event_t *event;
+    event_queue_t queue;
     xmlDocPtr doc;
-    xmlNodePtr node;
-    xmlNodePtr modules;
+    xmlNodePtr node, srcnode;
+    int len;
+    xmlChar *buff = NULL;
+    source_xml_t *snd;
+    source_xml_t *src_nodes = NULL;
+
+    event_queue_init (&queue);
+    _dump_stats_to_queue (&queue);
 
     doc = xmlNewDoc (XMLSTR("1.0"));
     node = xmlNewDocNode (doc, NULL, XMLSTR("icestats"), NULL);
     xmlDocSetRootElement(doc, node);
 
-    modules = module_container_get_modulelist_as_xml(global.modulecontainer);
-    xmlAddChild(node, modules);
 
-    node = _dump_stats_to_doc(node, flags, show_mount, client);
+    event = _get_event_from_queue(&queue);
+    while (event) {
+        if (event->source == NULL) {
+            xmlNewChild (node, NULL, XMLSTR(event->name), XMLSTR(event->value));
+        } else {
+            srcnode = _find_xml_node(event->source, &src_nodes, node);
+            xmlNewChild (srcnode, NULL, XMLSTR(event->name), XMLSTR(event->value));
+        }
 
-    return doc;
+        _free_event(event);
+        event = _get_event_from_queue(&queue);
+    }
+
+    xmlDocDumpMemory(doc, &buff, &len);
+    xmlFreeDoc(doc);
+    
+    client->respcode = 200;
+    bytes = sock_write(client->con->sock, "HTTP/1.0 200 OK\r\n"
+               "Content-Length: %d\r\n"
+               "Content-Type: text/xml\r\n"
+               "\r\n", len);
+    if (bytes > 0) client->con->sent_bytes += bytes;
+    else goto send_error;
+
+    bytes = client_send_bytes (client, buff, (unsigned)len);
+
+ send_error:
+    while (src_nodes) {
+        snd = src_nodes->next;
+        free(src_nodes->mount);
+        free(src_nodes);
+        src_nodes = snd;
+    }
+    if (buff) xmlFree(buff);
 }
 
-
 static int _compare_stats(void *arg, void *a, void *b)
 {
     stats_node_t *nodea = (stats_node_t *)a;
     stats_node_t *nodeb = (stats_node_t *)b;
 
-    (void)arg;
-
     return strcmp(nodea->name, nodeb->name);
 }
 
@@ -1151,8 +1064,6 @@ static int _compare_source_stats(void *arg, void *a, void *b)
     stats_source_t *nodea = (stats_source_t *)a;
     stats_source_t *nodeb = (stats_source_t *)b;
 
-    (void)arg;
-
     return strcmp(nodea->source, nodeb->source);
 }
 
@@ -1162,7 +1073,7 @@ static int _free_stats(void *key)
     free(node->value);
     free(node->name);
     free(node);
-
+    
     return 1;
 }
 
@@ -1220,7 +1131,7 @@ refbuf_t *stats_get_streams (void)
         }
         node = avl_get_next(node);
     }
-    thread_mutex_unlock(&_stats_mutex);
+    thread_mutex_unlock (&_stats_mutex);
     cur->len = STREAMLIST_BLKSIZE - remaining;
     return start;
 }
@@ -1246,7 +1157,7 @@ void stats_clear_virtual_mounts (void)
         {
             /* no source_t is reserved so remove them now */
             snode = avl_get_next (snode);
-            ICECAST_LOG_DEBUG("releasing %s stats", src->source);
+            DEBUG1 ("releasing %s stats", src->source);
             avl_delete (_stats.source_tree, src, _free_source_stats);
             continue;
         }
